@@ -1,5 +1,6 @@
 """This module implements the Aug class."""
-import uuid
+from uuid import UUID
+from random import getrandbits
 
 from rook.logger import logger
 from rook.processor.error import Error
@@ -21,20 +22,24 @@ class Aug(object):
 
     This class brings together the following elements:
     - location - specifies when to run the modification.
-    - extractor - specifies attributes to extract from the application's state before evaluating the modification.
     - condition - specifies an optional filter as to when to run the modification.
     - action - specifies the modification to preform.
     """
 
-    def __init__(self, aug_id, extractor, condition, action, global_rate_limit=AugRateLimiter(None, None),
+    TRACE_NAMESPACE_INSTANCE = TraceNamespace()
+    UTILS_NAMESPACE_INSTANCE = PythonUtilsNamespace()
+    STATE_NAMESPACE_INSTANCE = ProcessStateNamespace()
+
+    def __init__(self, aug_id, condition, action, global_rate_limit=AugRateLimiter(None, None),
                  max_aug_execution_time_ns=0, rate_limits_ns=None):
         """Build an Aug object from the individual elements."""
         self.aug_id = aug_id
-        self._extractor = extractor
         self._condition = condition
         self._action = action
         self._max_aug_time = max_aug_execution_time_ns
         self._enabled = True
+        self._executed = False
+        self._has_been_rate_limited = False
         self._global_rate_limiter = global_rate_limit
 
         if rate_limits_ns:
@@ -48,37 +53,35 @@ class Aug(object):
             return
 
         if output.is_user_messages_queue_full():
-            logger.warn("Skipping aug ({}) execution because the queue is full".format(self.aug_id))
+            output.send_output_queue_full_warning(self.aug_id)
             return
 
         now_ns = utils.get_most_accurate_time_stamp_nano_seconds()
         rate_limits_keys = self._allow(now_ns)
 
-        if Aug._enforce_rate_limits(rate_limits_keys):
+        if self._enforce_rate_limits(rate_limits_keys):
             return
 
         try:
-            if self._extractor:
-                self._extractor.execute(frame_namespace, extracted)
-
-            store = ContainerNamespace({})
-
             namespace = ContainerNamespace({
                 'frame': frame_namespace,
                 'stack': StackNamespace(frame_namespace),
                 'extracted': ContainerNamespace(extracted),
-                'store': store,
-                'temp': ContainerNamespace({}),
-                'python': PythonUtilsNamespace(),
-                'utils': PythonUtilsNamespace(),
-                'trace': TraceNamespace(),
-                'state': ProcessStateNamespace(),
+                'store': ContainerNamespace(),
+                'utils': Aug.UTILS_NAMESPACE_INSTANCE,
+                'trace': Aug.TRACE_NAMESPACE_INSTANCE,
+                'state': Aug.STATE_NAMESPACE_INSTANCE,
             })
 
             if not self._condition or self._condition.evaluate(namespace, extracted):
-                msg_id = uuid.uuid4().hex
+                msg_id = UUID(int=getrandbits(128), version=4).hex
+
+                if self._executed:
+                    logger.debug("Executing aug-\t%s (msg ID %s)", self.aug_id, msg_id)
+                else:
+                    logger.info("Executing aug for the first time-\t%s (msg ID %s)", self.aug_id, msg_id)
+                    self._executed = True
 
-                logger.info("Executing aug-\t%s (msg ID %s)", self.aug_id, msg_id)
                 self._action.execute(self.aug_id, msg_id, namespace, output)
         finally:
             duration = max(utils.get_most_accurate_time_stamp_nano_seconds() - now_ns, RateLimiter.MIN_RATE_LIMIT_VALUE_NS)
@@ -92,14 +95,17 @@ class Aug(object):
     def _allow(self, now_ns):
         return self._rate_limiter.allow(now_ns), self._global_rate_limiter.allow(now_ns)
 
-    @staticmethod
-    def _enforce_rate_limits(rate_limits_keys):
+    def _enforce_rate_limits(self, rate_limits_keys):
         if rate_limits_keys[0] is None:
-            UserWarnings.send_warning(Error(exc=RookRuleRateLimited()))
+            if not self._has_been_rate_limited:
+                UserWarnings.send_warning(Error(exc=RookRuleRateLimited()))
+                self._has_been_rate_limited = True
             return True
 
         if rate_limits_keys[1] is None:
-            UserWarnings.send_warning(Error(exc=RookGlobalRuleRateLimited()))
+            if not self._has_been_rate_limited:
+                UserWarnings.send_warning(Error(exc=RookGlobalRuleRateLimited()))
+                self._has_been_rate_limited = True
             return True
 
         return False
