@@ -12,7 +12,7 @@ import six, os
 from rook.factory import Factory
 from .aug_rate_limiter import AugRateLimiter
 
-from ..augs import actions, conditions, extractors, locations
+from ..augs import actions, conditions, locations
 from ..augs.aug import Aug
 
 
@@ -58,16 +58,12 @@ class AugFactory(Factory):
 
     def get_aug(self, configuration):
         """Returns an Aug object based on the given configuration."""
+        aug_id = None
         try:
             aug_id = configuration['id']
         except KeyError as exc:
             six.raise_from(RookAugInvalidKey('id', json.dumps(configuration)), exc)
 
-        if 'extractor' in configuration:
-            extractor = self._get_dynamic_class(configuration['extractor'])
-        else:
-            extractor = None
-
         condition = None
         conditional = configuration.get('conditional')
         if conditional:
@@ -87,7 +83,6 @@ class AugFactory(Factory):
         max_aug_execution_time = utils.milliseconds_to_int_nano_seconds(max_aug_execution_time)
 
         aug = Aug(aug_id=aug_id,
-                  extractor=extractor,
                   condition=condition,
                   action=action,
                   max_aug_execution_time_ns=max_aug_execution_time,
@@ -125,7 +120,6 @@ class AugFactory(Factory):
     def _load_dynamic_classes(self):
         """Load all dynamic classes into the factory."""
         self.register_methods(locations.__all__)
-        self.register_methods(extractors.__all__)
         self.register_methods(conditions.__all__)
         self.register_methods(actions.__all__)
 
