@@ -8,6 +8,7 @@ import six
 
 from rook.config import ImportServiceConfig
 from rook.serverless import on_lambda
+from rook.fork_utils import detect_multiprocessing
 
 _rook = None
 _debug = False
@@ -35,9 +36,14 @@ def start(token=None,
           proxy=None,
           fork=None,
           enable_monitor=None,
+          sources=None,
           **kwargs):
     global _rook, _debug, _throw_errors, _original_args, _fork
 
+    for k, v in os.environ.items():
+        if k.startswith("ROOKOUT_") and "LABELS" not in k:
+            os.environ[k] = v.strip().strip("'\"")  # First strip for whitespaces, then ' and "
+
     if _rook is not None:
         return
 
@@ -96,10 +102,20 @@ def start(token=None,
                                                                                          True) in _TRUE_VALUES
     async_start = os.environ.get('ROOKOUT_ASYNC_START', False) in _TRUE_VALUES
 
-    _fork = fork or os.environ.get('ROOKOUT_ENABLE_FORK', False) in _TRUE_VALUES
+    if fork is None and 'ROOKOUT_ENABLE_FORK' not in os.environ:
+        _fork = detect_multiprocessing()
+    else:
+        _fork = fork or os.environ.get('ROOKOUT_ENABLE_FORK', False) in _TRUE_VALUES
 
     _enable_monitor = enable_monitor or os.environ.get('ROOKOUT_ENABLE_MONITOR', True) in _TRUE_VALUES
 
+    try:
+        token = token.strip().strip("'\"")
+        proxy = proxy.strip().strip("'\"")
+        host_specified = host_specified.strip().strip("'\"")
+    except AttributeError:
+        pass
+
     try:
         from rook.exceptions import RookMissingToken, RookInvalidToken, RookVersionNotSupported, \
             RookCommunicationException, RookInvalidOptions, RookLoadError, RookOldServers, \
@@ -117,6 +133,22 @@ def start(token=None,
                     raise RookInvalidOptions('git_origin should be a String')
                 GitConfig.GIT_ORIGIN = git_origin
 
+            if sources is not None and not isinstance(sources, dict):
+                raise RookInvalidOptions('sources should be a dict')
+                
+            if not isinstance(sources, dict) and isinstance(os.environ.get('ROOKOUT_SOURCES'), six.string_types):
+                raw_sources = os.environ.get('ROOKOUT_SOURCES')
+                sources = {}
+                for source in raw_sources.split(';'):
+                    source = _normalize_string(source.strip(' \'"'))
+                    keyvalue = source.split('#')
+                    if len(keyvalue) == 2:
+                        key,value = keyvalue
+                        if key and value:                    
+                            sources[key] = value
+            if sources:                             
+                GitConfig.SOURCES = sources
+
             if log_file is not None:
                 if not isinstance(log_file, six.string_types):
                     raise RookInvalidOptions('log_file should be a String')
@@ -160,7 +192,7 @@ def start(token=None,
                                           proxy=proxy, throw_errors=throw_errors, log_level=log_level,
                                           log_to_stderr=log_to_stderr, log_file=log_file, git_commit=git_commit,
                                           git_origin=git_origin, tags=tags, labels=labels, async_start=async_start,
-                                          use_import_hook=use_import_hook, fork=_fork, enable_monitor=_enable_monitor)
+                                          use_import_hook=use_import_hook, fork=_fork, enable_monitor=_enable_monitor, sources=sources)
 
             if (host == "staging.cloud.agent.rookout.com") or (host == "cloud.agent.rookout.com"):
                 raise RookOldServers()
