@@ -23,6 +23,11 @@ from rook.protobuf import variant_pb2, variant2_pb2
 
 from ..user_warnings import UserWarnings
 
+try:
+    from bson import ObjectId, decode
+    from bson.raw_bson import RawBSONDocument
+except ImportError:
+    pass
 
 class NamespaceSerializer2(NamespaceSerializerBase):
     def __init__(self):
@@ -115,6 +120,8 @@ class NamespaceSerializer2(NamespaceSerializerBase):
             self._dump_primitive(str(obj), original_type, variant, config.max_string)
         elif NamespaceSerializer2.is_protobuf_obj(obj):
             self._dump_protobuf(obj, variant, current_depth, config, log_errors)
+        elif NamespaceSerializer2.is_bson(obj):
+            self._dump_bson(obj, variant, current_depth, config, log_errors)
         elif hasattr(obj, '__dict__'):
             self._dump_user_class(obj, variant, current_depth, config, log_errors)
         else:
@@ -277,6 +284,15 @@ class NamespaceSerializer2(NamespaceSerializerBase):
                 except Exception:  # for now we just ignore errors when dumping protobuf
                     pass
 
+    def _dump_bson(self, obj, variant, current_depth, config, log_errors):
+        if isinstance(obj, ObjectId):
+            return self._dump_primitive(str(obj), str(type(obj)), variant, config.max_string)
+
+        if isinstance(obj, RawBSONDocument):
+            return self._dump_dictionary(decode(obj.raw), variant, current_depth, config, log_errors)
+
+        return None
+
     def _dump_exception(self, exc, variant, current_depth, config, log_errors):
         self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_OBJECT)
 
