@@ -1,14 +1,17 @@
 import datetime
 import decimal
 from types import FunctionType, MethodType, ModuleType
+from google.protobuf.reflection import GeneratedProtocolMessageType
+from google.protobuf.internal import api_implementation
+
+if api_implementation.Type() == "upb":
+    import google
 
 import six
 
 if six.PY2:
     from types import TypeType
 
-from google.protobuf.descriptor import Descriptor as google_descriptor
-
 try:
     import numpy
 except ImportError:
@@ -24,6 +27,12 @@ try:
 except ImportError:
     multidict = None
 
+try:
+    from bson import ObjectId
+    from bson.raw_bson import RawBSONDocument
+except ImportError:
+    ObjectId = None
+    RawBSONDocument = None
 
 class NamespaceSerializerBase(object):
     BUILTIN_ATTRIBUTES_IGNORE = {
@@ -68,15 +77,15 @@ class NamespaceSerializerBase(object):
     def get_estimated_pending_bytes(self):
         return self.estimated_pending_bytes
 
-    def _get_string_index_in_cache(self, str):
-        if str in self.string_cache:
-            return self.string_cache[str]
+    def _get_string_index_in_cache(self, string_to_cache):
+        index = self.string_cache.get(string_to_cache, None)
+        if index is None:
+            index = len(self.string_cache)
+            # We estimate each character is one byte in utf-8 and overhead is 5 bytes
+            self.estimated_pending_bytes += len(string_to_cache) + 5
+            self.string_cache[string_to_cache] = index
 
-        current_size = len(self.string_cache)
-        # We estimate each character is one byte in utf-8 and overhead is 5 bytes
-        self.estimated_pending_bytes += len(str) + 5
-        self.string_cache[str] = current_size
-        return current_size
+        return index
 
     @staticmethod
     def _get_object_width(obj):
@@ -87,16 +96,17 @@ class NamespaceSerializerBase(object):
             object_width += len(obj.__slots__)
         return object_width
 
-
-    @staticmethod
-    def normalize_string(obj):
-        if six.PY2:
+    if six.PY2:
+        @staticmethod
+        def normalize_string(obj):
             if isinstance(obj, str):
                 return unicode(obj, errors="replace")
             else:
                 return unicode(obj)
-        else:
-            return str(obj)
+    else:
+        @staticmethod
+        def normalize_string(obj):
+            return obj
 
     @staticmethod
     def is_numpy_obj(obj):
@@ -106,11 +116,14 @@ class NamespaceSerializerBase(object):
         return not isinstance(obj, six.MovedModule) and isinstance(obj, numpy.generic)
 
     @staticmethod
-    def is_torch_obj(obj):
-        if not torch:
+    def is_torch_obj(obj, obj_type=None):
+        if torch is None:
             return False
 
-        module = getattr(type(obj), '__module__', None)
+        if obj_type is None:
+            obj_type = type(obj)
+
+        module = getattr(obj_type, '__module__', None)
         if not module:
             return False
 
@@ -118,14 +131,35 @@ class NamespaceSerializerBase(object):
 
     @staticmethod
     def is_multidict_obj(obj):
-        if not multidict or isinstance(obj, six.MovedModule) or not hasattr(multidict, '_multidict'):
+        if not multidict:
             return False
 
-        return isinstance(obj, (multidict._multidict.MultiDict, multidict._multidict.CIMultiDict, multidict._multidict.MultiDictProxy, multidict._multidict.CIMultiDictProxy))
+        return isinstance(obj, (multidict.MultiDict, multidict.CIMultiDict,
+                                multidict.MultiDictProxy,
+                                multidict.CIMultiDictProxy))
 
     @staticmethod
     def is_protobuf_obj(obj):
-        if not isinstance(obj, six.MovedModule) and hasattr(obj, "DESCRIPTOR"):
-            return isinstance(obj.DESCRIPTOR, google_descriptor)
+        klass = getattr(obj, '__class__', None)
+        if api_implementation.Type() == "upb":
+            if klass is not None and isinstance(klass, google._upb._message.MessageMeta):
+                return True
+        else:
+            if klass is not None and isinstance(klass, GeneratedProtocolMessageType):
+                return True
+
+        return False
+
+    @staticmethod
+    def is_bson(obj):
+        if not ObjectId or not RawBSONDocument:
+            return False
+
+        klass = getattr(obj, '__class__', None)
+        if klass is not None and isinstance(obj, ObjectId):
+            return True
+
+        if klass is not None and isinstance(obj, RawBSONDocument):
+            return True
 
         return False
