@@ -41,7 +41,13 @@ except NameError:
 
 
 class Marker(object):
-    pass
+    def write(self, namespace, value):
+        raise RookOperationReadOnly(type(self))
+
+
+class ObjectMarker(Marker):
+    def read(self, namespace, create):
+        return self
 
 
 class FunctionOperation(Marker):
@@ -52,9 +58,6 @@ class FunctionOperation(Marker):
     def read(self, namespace, create):
         return namespace.call_method(self.function_name, self.function_arguments)
 
-    def write(self, namespace, value):
-        raise RookOperationReadOnly(type(self))
-
 
 class AttributeOperation(Marker):
     def __init__(self, name):
@@ -86,11 +89,70 @@ class LookupOperation(Marker):
     def read(self, namespace, create):
         return namespace.read_key(self.name)
 
+
+class OperationList(Marker):
+    def __init__(self, expression, list):
+        self.expression = expression
+        self.list = list
+
+    def read(self, namespace, create):
+        try:
+            for op in self.list:
+                namespace = op.read(namespace, False)
+
+            if isinstance(namespace, PythonObjectNamespace) and namespace.obj and not isinstance(namespace.obj, PRIMITIVE_VALUES):
+                 # returning this class that mark its non primitive for additional information
+                 # this will help us in case of non True result when comparing this object
+                return NonPrimitiveNamespaceResult(namespace, self.expression)
+
+            return NamespaceResult(namespace)
+        except Exception as e:
+            return ToolException_(e)
+
     def write(self, namespace, value):
-        raise RookOperationReadOnly(type(self))
+        size = len(self.list)
+        if size == 0:
+            raise RookInvalidArithmeticPath(self.expression)
+
+        # execute the operation chain.
+        i = 0
+        for _ in range(i, size - 1):
+            namespace = self.list[i].read(namespace, True)
+            i += 1
+
+        return self.list[i].write(namespace, value)
 
 
-class ToolException_(Marker):
+class FlatOpt(Marker):
+    def __init__(self, flat_elements):
+        self.flat_elements = flat_elements
+
+    def read(self, namespace, create):
+        flat_elements_copy = list(self.flat_elements)
+        while len(flat_elements_copy) > 1:
+            stop = False
+            for level in range(len(allLevels)):
+                for i in range(1, len(flat_elements_copy), 2):
+                    current_opt = flat_elements_copy[i]
+                    if current_opt.level == level:
+                        result = current_opt.execute_operation(namespace, flat_elements_copy[i - 1], flat_elements_copy[i + 1])
+                        flat_elements_copy = flat_elements_copy[:i - 1] + [result] + flat_elements_copy[i + 2:]
+                        stop = True
+                        break
+                if stop:
+                    break
+
+        return flat_elements_copy[0]
+
+    def get_obj(self, namespace):
+        result = self.read(namespace, False)
+        if result.get_obj:
+            return result.get_obj()
+        else:
+            return result
+
+
+class ToolException_(ObjectMarker):
     def __init__(self, exc):
         self.obj = exc
 
@@ -98,7 +160,7 @@ class ToolException_(Marker):
         return str(self.obj)
 
 
-class NamespaceResult(Marker):
+class NamespaceResult(ObjectMarker):
     def __init__(self, namespace):
         self.obj = namespace
         if isinstance(namespace, PythonObjectNamespace):
@@ -128,7 +190,7 @@ class NonPrimitiveNamespaceResult(NamespaceResult):
         return 'NonPrimitiveNamespaceResult'
 
 
-class Text(Marker):
+class Text(ObjectMarker):
     def __init__(self, string):
         self.obj = string
         self.text = string
@@ -140,7 +202,7 @@ class Text(Marker):
         return self.obj
 
 
-class TextResult(Marker):
+class TextResult(ObjectMarker):
     def __init__(self, string):
         self.obj = string
         self.text = string
@@ -152,7 +214,7 @@ class TextResult(Marker):
         return self.obj
 
 
-class Array(Marker):
+class Array(ObjectMarker):
     def __init__(self, l, string):
         self.obj = l
         self.text = string
@@ -164,7 +226,7 @@ class Array(Marker):
         return self.obj
 
 
-class FloatNumber(Marker):
+class FloatNumber(ObjectMarker):
     def __init__(self, num):
         self.obj = float(num)
         self.text = num
@@ -176,7 +238,7 @@ class FloatNumber(Marker):
         return self.obj
 
 
-class Number(Marker):
+class Number(ObjectMarker):
     def __init__(self, num):
         self.obj = int(num)
         self.text = num
@@ -188,7 +250,7 @@ class Number(Marker):
         return self.obj
 
 
-class Char(Marker):
+class Char(ObjectMarker):
     def __init__(self, char):
         self.obj = char
         self.text = "'" + char + "'"
@@ -200,7 +262,7 @@ class Char(Marker):
         return self.obj
 
 
-class Bool(Marker):
+class Bool(ObjectMarker):
     def __init__(self, boolean):
         self.obj = (boolean == 'true') or (boolean == 'True')
         self.text = boolean
@@ -212,7 +274,7 @@ class Bool(Marker):
         return self.obj
 
 
-class Undefined(Marker):
+class Undefined(ObjectMarker):
     def __init__(self):
         self.obj = None
         self.text = None
@@ -225,7 +287,7 @@ class Undefined(Marker):
         return self.obj
 
 
-class Opt(Marker):
+class Opt(ObjectMarker):
     _opMap = {
         "+": lambda a, b: a + b,
         "-": lambda a, b: a - b,
@@ -279,7 +341,10 @@ class Opt(Marker):
         else:
             return element.text
 
-    def execute_operation(self, a, b):
+    def execute_operation(self, namespace, a, b):
+        a = a.read(namespace, False)
+        b = b.read(namespace, False)
+
         if self.level == 3:
             first_element_string = Opt.get_element_string(a)
 
@@ -311,6 +376,11 @@ class Opt(Marker):
             return Bool('False')
         else:
             try:
+                if isinstance(a, ToolException_):
+                    return a
+                if isinstance(b, ToolException_):
+                    return b
+
                 result = Opt._opMap[self.opt](a.get_obj(), b.get_obj())
             except Exception as e:
                 operation = a.to_string() + self.opt + b.to_string()
@@ -333,10 +403,8 @@ class Opt(Marker):
 
 
 class Actions(object):
-    def __init__(self, namespace):
-        self.namespace = None
-        self.operations = []
-        self.namespace = namespace
+    def __init__(self, expression):
+        self.expression = expression
 
     @staticmethod
     def make_lookup_operation(input, start, end, elements):
@@ -369,7 +437,9 @@ class Actions(object):
         if isinstance(elements[4], ToolException_):
             return elements[4]
 
-        args = elements[4].text
+        args = ""
+        if isinstance(elements[4], ObjectMarker):
+            args = elements[4].text
         return FunctionOperation(function_name, args)
 
     @staticmethod
@@ -387,27 +457,19 @@ class Actions(object):
         # For further explanation, check ArithmeticPath.peg
 
         try:
-            self.operations = []
-            self.operations.append(elements[1])
+            operations = [elements[1]]
 
             for e in elements[2].elements:
-                self.operations.append(e)
+                operations.append(e)
 
             # Check if we have some exceptions in the operations chain - might happen if function parsing failed.
-            for op in self.operations:
+            for op in operations:
                 if isinstance(op, ToolException_):
                     return op
 
-            result = elements[1].read(self.namespace, False)
-            for e in elements[2].elements:
-                result = e.read(result, False)
+            # As part of optimization, we don't actually execute the namespace here
+            return OperationList(self.expression, operations)
 
-            if isinstance(result, PythonObjectNamespace) and result.obj and not isinstance(result.obj, PRIMITIVE_VALUES):
-                # returning this class that mark its non primitive for additional information
-                # this will help us in case of non True result when comparing this object
-                return NonPrimitiveNamespaceResult(result, input[start:end])
-
-            return NamespaceResult(result)
         except Exception as e:
             return ToolException_(e)
 
@@ -444,20 +506,7 @@ class Actions(object):
                 return n.elements[1]
             flat_elements.append(n.elements[1])
 
-        while len(flat_elements) > 1:
-            stop = False
-            for level in range(len(allLevels)):
-                for i in range(1, len(flat_elements), 2):
-                    current_opt = flat_elements[i]
-                    if current_opt.level == level:
-                        result = current_opt.execute_operation(flat_elements[i - 1], flat_elements[i + 1])
-                        flat_elements = flat_elements[:i - 1] + [result] + flat_elements[i + 2:]
-                        stop = True
-                        break
-                if stop:
-                    break
-
-        return flat_elements[0]
+        return FlatOpt(flat_elements)
 
     @staticmethod
     def make_opt(input, start, end, elements):
@@ -537,36 +586,19 @@ class ArithmeticPath(object):
         if self.expression == "":
             raise RookInvalidArithmeticPath(self.expression)
 
-    def read_from(self, root_namespace):
-        context = Actions(root_namespace)
+        self.actions = Actions(self.expression)
 
         try:
-            result = parse(self.expression, actions=context)
+            self.operations = parse(self.expression, actions=self.actions)
         except ParseError as e:
             raise RookInvalidArithmeticPath(self.expression, e)
+
+    def read_from(self, root_namespace):
+        result = self.operations.read(root_namespace, False)
         return self.normalize_result(result, self.expression)
 
     def write_to(self, root_namespace, value):
-        context = Actions(root_namespace)
-
-        try:
-            # initialize operations chain - by parsing the expression - ignore return value
-            parse(self.expression, actions=context)
-        except ParseError as e:
-            raise RookInvalidArithmeticPath(self.expression, e)
-
-        # execute the operation chain.
-        size = len(context.operations)
-        if size == 0:
-            raise RookInvalidArithmeticPath(self.expression)
-
-        i = 0
-        namespace = root_namespace
-        for _ in range(i, size - 1):
-            namespace = context.operations[i].read(namespace, True)
-            i += 1
-
-        context.operations[i].write(namespace, value)
+        self.operations.write(root_namespace, value)
 
     def normalize_result(self, result, expression):
         if isinstance(result, Array):
